/*
 *
 * Copyright (C) 2004 Mekensleep
 *
 *	Mekensleep
 *	24 rue vieille du temple
 *	75004 Paris
 *       licensing@mekensleep.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *  Loic Dachary <loic@gnu.org>
 *
 */

#include <cal3d/coresubmesh.h>
#include <cal3d/coremesh.h>

#include <osgCal/DynamicCalCoreModel>

DynamicCalCoreModel::DynamicCalCoreModel() {}
DynamicCalCoreModel::~DynamicCalCoreModel() {}

// store a map filename => id + re-use id when possible
int DynamicCalCoreModel::loadCoreAnimation(const std::string& strFilename, const std::string& name)
{
  int id = getCoreAnimationId(name);
  if(id >= 0)
    {
      // the core skeleton has to be loaded already
      if(m_pCoreSkeleton == 0)
	{
	  CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
	  return -1;
	}
      if(m_vectorCoreAnimation[id])
	{
	  CalError::setLastError(CalError::INDEX_BUILD_FAILED, __FILE__, __LINE__);
	  return -1;
	}
      CalCoreAnimation *pCoreAnimation = CalLoader::loadCoreAnimation(strFilename);
      if(pCoreAnimation == 0) return -1;
      m_vectorCoreAnimation[id] = pCoreAnimation;
    }
  else
    {
      id = CalCoreModel::loadCoreAnimation(strFilename);
      if(id >= 0) {
	m_animation2id[name] = id;
	m_animationName2Filename[name] = strFilename;
      }
    }

  return id;
}

int DynamicCalCoreModel::getCoreAnimationId(const std::string& name)
{
  if(m_animation2id.find(name) != m_animation2id.end())
    return m_animation2id[name];
  else
    return -1;
}

int DynamicCalCoreModel::unloadCoreAnimation(const std::string& name)
{
  int id = getCoreAnimationId(name);
  if(id >= 0)
    return unloadCoreAnimation(id);
  else
    return -1;
}

int DynamicCalCoreModel::unloadCoreAnimation(int coreAnimationId)
{
  if((coreAnimationId < 0) || (coreAnimationId >= (int)m_vectorCoreAnimation.size()))
  {
    CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
    return -1;
  }

  delete m_vectorCoreAnimation[coreAnimationId];
  m_vectorCoreAnimation[coreAnimationId] = 0;

  return coreAnimationId;
}

// store a map filename => id + re-use id when possible
int DynamicCalCoreModel::loadCoreMesh(const std::string& strFilename, const std::string& name)
{
  // the core skeleton has to be loaded already
  if(m_pCoreSkeleton == 0)
    {
      CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
      return -1;
    }

  CalCoreMesh *pCoreMesh = CalLoader::loadCoreMesh(strFilename);
  if(pCoreMesh == 0) return -1;
  int count = pCoreMesh->getCoreSubmeshCount();
  for(int submeshId = 0; submeshId < count; submeshId++) {
    CalCoreSubmesh* submesh = pCoreMesh->getCoreSubmesh(submeshId);
    std::vector<std::vector<CalCoreSubmesh::TextureCoordinate> >& uv = submesh->getVectorVectorTextureCoordinate();
    for(unsigned int n = 0; n < uv.size(); n++)
      for(unsigned int iuv = 0; iuv < uv[n].size(); iuv++)
	uv[n][iuv].v=1.0-uv[n][iuv].v;
  }

  int id = getCoreMeshId(name);
  if(id >= 0)
    {
      if(m_vectorCoreMesh[id])
	{
	  CalError::setLastError(CalError::INDEX_BUILD_FAILED, __FILE__, __LINE__, "already allocated");
	  return -1;
	}
      m_vectorCoreMesh[id] = pCoreMesh;
    }
  else
    {
      id = addCoreMesh(pCoreMesh);
      if(id >= 0) {
	m_mesh2id[name] = id;
	m_meshName2Filename[name] = strFilename;
      }
    }

  return id;
}

int DynamicCalCoreModel::getCoreMeshId(const std::string& name)
{
  if(m_mesh2id.find(name) != m_mesh2id.end())
    return m_mesh2id[name];
  else
    return -1;
}

int DynamicCalCoreModel::getCoreMeshId(CalCoreMesh* coreMesh)
{
  int coreMeshId = -1;
  for(std::vector<CalCoreMesh *>::iterator coreMeshIterator = m_vectorCoreMesh.begin();
      coreMeshIterator != m_vectorCoreMesh.end();
      coreMeshIterator++) {
    coreMeshId++;
    if(*coreMeshIterator == coreMesh)
      return coreMeshId;
  }
  return -1;
}

int DynamicCalCoreModel::unloadCoreMesh(const std::string& name)
{
  int id = getCoreMeshId(name);
  if(id >= 0)
    return unloadCoreMesh(id);
  else
    return -1;
}

int DynamicCalCoreModel::unloadCoreMesh(int coreMeshId)
{
  if((coreMeshId < 0) || (coreMeshId >= (int)m_vectorCoreMesh.size()))
  {
    CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
    return -1;
  }

  delete m_vectorCoreMesh[coreMeshId];
  m_vectorCoreMesh[coreMeshId] = 0;

  return coreMeshId;
}

// store a map filename => id + re-use id when possible
int DynamicCalCoreModel::loadCoreMaterial(const std::string& strFilename, const std::string& name)
{
  int coreMaterialId = CalCoreModel::loadCoreMaterial(strFilename);
  if(coreMaterialId >= 0) {
    m_material2id[name] = coreMaterialId;
    m_materialName2Filename[name] = strFilename;
  }

  return coreMaterialId;
}

int DynamicCalCoreModel::getCoreMaterialId(const std::string& name)
{
  //  std::cerr << "DynamicCalCoreModel::getCoreMaterialId: " << name;
  if(m_material2id.find(name) != m_material2id.end()) {
    //    std::cerr << " " << m_material2id[name] << std::endl;
    return m_material2id[name];
  } else {
    //    std::cerr << " none found" << std::endl;
    return -1;
  }
}
